"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AliasHelper = void 0;
exports.createAliasInlineAction = createAliasInlineAction;
exports.createAliasSaveActionNewFile = createAliasSaveActionNewFile;
const os = __importStar(require("os"));
const vscode_languageserver_1 = require("vscode-languageserver");
const tree_sitter_1 = require("../utils/tree-sitter");
const exec_1 = require("../utils/exec");
const path_1 = require("path");
const action_kinds_1 = require("./action-kinds");
const translation_1 = require("../utils/translation");
function extractFunctionName(node) {
    const children = node.children;
    if (children.length < 2)
        return '';
    const nameNode = children[1];
    if (!nameNode)
        return '';
    const name = nameNode.text.split('=')[0]?.toString() || '';
    return name.trim();
}
async function createAliasInlineAction(doc, node) {
    const aliasCommand = node.text;
    const funcName = extractFunctionName(node);
    if (!funcName) {
        return undefined;
    }
    const stdout = await (0, exec_1.execAsyncF)(`${aliasCommand} && functions ${funcName} | tail +2 | fish_indent`);
    const edit = vscode_languageserver_1.TextEdit.replace((0, tree_sitter_1.getRange)(node), `\n${stdout}\n`);
    return {
        title: `Convert alias '${funcName}' to inline function`,
        kind: action_kinds_1.SupportedCodeActionKinds.RefactorExtract,
        edit: {
            changes: {
                [doc.uri]: [edit],
            },
        },
        isPreferred: true,
    };
}
function createVersionedDocument(uri) {
    return vscode_languageserver_1.VersionedTextDocumentIdentifier.create(uri, 0);
}
function createFunctionFileEdit(functionUri, content) {
    return vscode_languageserver_1.TextDocumentEdit.create(createVersionedDocument(functionUri), [vscode_languageserver_1.TextEdit.insert({ line: 0, character: 0 }, content)]);
}
function createRemoveAliasEdit(document, node) {
    return vscode_languageserver_1.TextDocumentEdit.create(createVersionedDocument(document.uri), [vscode_languageserver_1.TextEdit.del((0, tree_sitter_1.getRange)(node))]);
}
async function createAliasSaveActionNewFile(doc, node) {
    const aliasCommand = node.text;
    const funcName = extractFunctionName(node);
    const functionContent = await (0, exec_1.execAsyncF)(`${aliasCommand} && functions ${funcName} | tail +2 | fish_indent`);
    const functionPath = (0, path_1.join)(os.homedir(), '.config', 'fish', 'functions', `${funcName}.fish`);
    const functionUri = (0, translation_1.pathToUri)(functionPath);
    const createFileAction = vscode_languageserver_1.CreateFile.create(functionUri, {
        ignoreIfExists: false,
        overwrite: true,
    });
    const workspaceEdit = {
        documentChanges: [
            createFileAction,
            createFunctionFileEdit(functionUri, functionContent),
            createRemoveAliasEdit(doc, node),
        ],
    };
    return {
        title: `Convert alias '${funcName}' to function in file: ~/.config/fish/functions/${funcName}.fish`,
        kind: action_kinds_1.SupportedCodeActionKinds.RefactorExtract,
        edit: workspaceEdit,
        isPreferred: false,
    };
}
exports.AliasHelper = {
    extractFunctionName,
    createAliasInlineAction,
    createAliasSaveActionNewFile,
};
